########################
# Lecture 9
# Panel Data
# POLS 509: The Linear Model
# Dr. Justin Esarey
########################

#install.packages(c("pcse","plm","Hmisc","lme4","arm"), dep=T)

library(foreign)
library(pcse)  # a package enabling the estimation of Panel-Corrected Standard Errors
library(plm)	# the panel linear models library.
library(Hmisc)


read.dta("505-pooled-se.dta")->data    

# Data prep
############

data<-subset(data, data$year>1977 & data$year<2001)
data<-subset(data, data$stname!="District of Columbia")

# per capita police spending, state + local
data$policePC = data$policeex1/data$totalpop
# per capita welfare spending, state + local
data$welfarePC = data$welfare1/data$totalpop
# per capita education spending, state + local
data$edspendPC = data$edspend1/data$totalpop
# log population
data$logpop = log(data$totalpop)
# log per capita income
data$logpcinc = log(data$PInccapC)

attach(data)


# Consider first a basic regression with no accounting for unit heterogeneity. 
model<-lm(murder~adperI+policePC+edspendPC+logpcinc+logpop)
summary(model)

# look at Panel-Corrected Standard Errors
model.pcse<-pcse(model, groupN=data$stfips, groupT=data$year)
summary(model.pcse)


# read data into PLM package
data.plm<-pdata.frame(data, index=c("stfips", "year"), drop.index=T, row.names=T)

# same ols model as before
model.plm <- model.fe.plm<-plm(murder~adperI+policePC+edspendPC+logpcinc+logpop, data=data.plm, model="pooling")
summary(model.plm)

# add dummy variables for each year
model.plm.2 <- model.fe.plm<-plm(murder~adperI+policePC+edspendPC+logpcinc+logpop, data=data.plm, effect="time", model="within")
summary(model.plm.2)
# for comparison: basic lm with same idea
mod.comp <-lm(murder~adperI+policePC+edspendPC+logpcinc+logpop+as.factor(year), data=data)
summary(mod.comp)


# heteroskedasticity-robust SEs, HC3 variant
vcvhc3 <- vcovHC(model.plm, method="arellano", type="HC3")
sqrt(diag(vcvhc3))

# add clustering by time
vcvhc3.time <- vcovHC(model.plm, method="arellano", type="HC3", cluster="time")
sqrt(diag(vcvhc3.time))


# alternate method
vcovClusterOneWay <- function(fm, cluster, dfcw=1){
  # R-codes (www.r-project.org) for computing
  # clustered-standard errors. Mahmood Arai, Jan 26, 2008.
  # The arguments of the function are:
  # fitted model, cluster1 and cluster2
  # You need to install libraries `sandwich' and `lmtest'
  # reweighting the var-cov matrix for the within model
  library(sandwich);library(lmtest)
  M <- length(unique(cluster))
  N <- length(cluster)
  K <- fm$rank
  dfc <- (M/(M-1))*((N-1)/(N-K))
  uj <- apply(estfun(fm),2, function(x) tapply(x, cluster, sum));
  vcovCL <- dfc*sandwich(fm, meat=crossprod(uj)/N)*dfcw
  vcovCL
}

cluster.vcv <- vcovClusterOneWay(mod.comp, cluster=year)
cluster.vcv[1:6,1:6]
sqrt(diag(cluster.vcv[1:6,1:6]))
# this is the same as...
vcvhc0.time <- vcovHC(model.plm, method="arellano", type="HC0", cluster="time")
sqrt(diag(vcvhc0.time)) # need a "small sample" degrees of freedom correction
M <- length(unique(data$year))
N <- length(data$year)
K <- mod.comp$rank
dfc <- (M/(M-1))*((N-1)/(N-K))
sqrt(diag(dfc*vcvhc0.time))

# clustering by state, instead
vcvhc3.st <- vcovHC(model.plm, method="arellano", type="HC3", cluster="group")
sqrt(diag(vcvhc3.st))


library(clusterSEs)
# requires GLM-type models
model<-glm(murder~adperI+policePC+edspendPC+logpcinc+logpop, dat=data)
summary(model)


cluster.bs(mod=model, dat=data, cluster = ~ year)
cluster.bs(mod=model, dat=data, cluster = ~ stfips)

cluster.im(mod=model, dat=data, cluster = ~ year)
cluster.im(mod=model, dat=data, cluster = ~ stfips)

# dummies for state, clustering over year
model.2<-glm(murder~adperI+policePC+edspendPC+logpcinc+logpop+as.factor(stfips), dat=data)
summary(model.2)
cluster.bs(mod=model.2, dat=data, cluster = ~ year)